#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "homemade_speedstep.h"

int main(int argc, char** argv){
  /* check for no arguments or help */
  if (argc == 1 || comparestrings(argv[1], "-h") == 0
      || comparestrings(argv[1], "--help") == 0){
    fprintf(stderr, "usage: homemade_speedstep <max temp> <min temp>\n");
    fprintf(stderr, "       homemade_speedstep [-h | --help | -v | --version]\n");
    fprintf(stderr, "       -h | --help      prints out this help message\n");
    fprintf(stderr, "       -v | --version   prints out version number\n");
    return 0;
  }
  /* print version number if asked to and exit */
  if (comparestrings(argv[1], "-v") == 0
      || comparestrings(argv[1], "--version") == 0){
    fprintf(stderr, VERSION"\n");
    return 0;
  }
  /* check for correct ammount of arguments */
  if (argc != 3){
    fprintf(stderr, "error: wrong ammount of arguments given\n");
    fprintf(stderr, "       see homemade_speedstep --help\n");
    return 1;
  }
  /* set maxtemp and mintemp TODO: check for integer */
  const int maxtemp = stringtoint(argv[1], stringlength(argv[1]));
  const int mintemp = stringtoint(argv[2], stringlength(argv[2]));
  /* check that the values are, at the very least, different
     and that max is higher than min */
  if (maxtemp <= mintemp){
    fprintf(stderr, "error: maxtemp must be higher than mintemp\n");
    fprintf(stderr, "       see homemade_speedstep --help\n");
    return 2;
  }
  /* try to open scaling_available_frequencies, exit on fail */
  FILE *freqsfile;
  if ((freqsfile = fopen(FREQS, "r")) == NULL){
    FILEERROR("read", "scaling_available_frequencies");
    return 5;
  }
  /* assign content of freqsfile to a temporary char array */
  char* freqsfilec = malloc(sizeof(char) * 200);
  getfilec(freqsfile, freqsfilec);
  /* close file since we no loger use it */
  fclose(freqsfile);
  /* count the number of steps (should probably figure out a better way of doing this */
  int stepc = countblanks(freqsfilec);
  /* create a char ** array with pointers to freq steps */
  char ** steps = malloc(sizeof(char*) * stepc);
  int* index = malloc(sizeof(int*));
  *index = 0;
  int i;
  for(i = 0; i < stepc; i++){
    *index = splitstr(steps, freqsfilec, *index, i);
  }
  /* free memory */
  free(index);
  free(freqsfilec);
  /* try to open scaling_maximum_frequency, exit on fail */
  FILE* curmaxfile;
  if ((curmaxfile = fopen(MAXSPDPREFIX"0"MAXSPDSUFFIX, "r")) == NULL){
    FILEERROR("read", "scaling_maximum_frequency");
    return 6;
  }
  /* assign content of curmaxfile to a temporary char array */
  char *curmaxfreq = malloc(sizeof(char) * 20);
  getfilec(curmaxfile, curmaxfreq);
  /* close file since we're done using it */
  fclose(curmaxfile);
  newlinetonullbyte(curmaxfreq);
  /* figure out what step we're on */
  int curstep = -1;
  for(i = 0; i < stepc; i++){
    if (comparestrings(steps[i], curmaxfreq) == 0){
      curstep = i;
      break;
    }
  }
  /* free memory */
  free(curmaxfreq);
  /* check if we didn't find our current step, exit on fail */
  if (curstep == -1){
    fprintf(stderr, "error: couldn't figure out the current frequency\n");
    fprintf(stderr, "       check that you have support for cpufreq\n");
    fprintf(stderr, "       in your kernel and that the program is\n");
    fprintf(stderr, "       running as root\n");
    return 7;
  }
  /* these two go on the stack because we're going to need it throughout
     the program's lifetime and also because they're small */
  int timepassed = 0;
  int temperature;
  FILE* curtempfile;
  /* setup is ready, time to run the main loop */
  while((curtempfile = fopen(CURTEMPPATH, "r")) != NULL){
  /* we don't want timepassed to grow infinitely */
    if (timepassed < 61){
      timepassed++;
    }
    /* 7 chars should be enough to store temp, but I put two more just in case
       if temperature is above 9999999ºC there's bigger problems than memory */
    char* char_temperature = malloc(sizeof(char) * 8);
    getfilec(curtempfile, char_temperature);
    /* close file since we're done using it */
    fclose(curtempfile);
    newlinetonullbyte(char_temperature);
    temperature = stringtoint(char_temperature, stringlength(char_temperature));
    /* free memory */
    free(char_temperature);
    /* check if current temp is higher than max and step down */
    if (temperature >= maxtemp && curstep < stepc - 1){
      curstep++;
      if (changefreq(THREADS, steps[curstep]) != 0){
          FILEERROR("write to", "scaling_max_freq");
        return 8;
      }
      timepassed = 0;
    }
    /* check if current temp is lower than min and step up */
    if (temperature <= mintemp && curstep > 0){
      curstep--;
      if (changefreq(THREADS, steps[curstep]) != 0){
          FILEERROR("write to", "scaling_max_freq");
        return 8;
      }
      timepassed = 0;
    }
    /* check if a minute passed without step changes and temp is below minmax average */
    if (timepassed > 60 && temperature <= (mintemp+maxtemp)/2 && curstep > 0){
      curstep--;
      if (changefreq(THREADS, steps[curstep]) != 0){
          FILEERROR("write to", "scaling_max_freq");
        return 8;
      }
      timepassed = 0;
    }
    sleep(POLLING_TIME);
  }
  /* something went wrong if we're here, make sure to free last dynamic memory */
  free(steps);
  FILEERROR("read", "temp");
  return 9;
}
